; $Id: live_plot.pro,v 1.5 1997/04/16 20:19:16 billo Exp $
;
; Copyright (c) 1996-1997, Research Systems, Inc.  All rights reserved.
;       Unauthorized reproduction prohibited.
;
;+
; NAME:
;       LIVE_PLOT
;
; PURPOSE:
;       Entry point to LIVE_PLOT
;
; CALLING SEQUENCE:
;       LIVE_PLOT, yVector
;
; OPTIONAL KEYWORDS:
;       INDEPENDENT=xVector		   ; The independent vector [optional]
;       STYLE=style				   ; A style name (string) [optional]
;		LINE=line				   ; Set to get a line plot (default)[optional]
;		SCATTER=scatter			   ; Set to get a scatter plot [optional]
;		HISTOGRAM=histogram		   ; Set to get a histogram plot [optional]
;		POLAR=polar				   ; Set to get a polar plot [optional]
;       INDEXED_COLOR=indexedColor ; Set to use indexed color [optional]
;
; DEVELOPMENT NOTES:
;		-
;
;------------------------------------------------------------------------------
pro Live_Plot, y, _EXTRA=extra

	ON_ERROR,2

	WIDGET_CONTROL, /HOURGLASS

	; Determine if the L_PLOT routine has been resolved
	;
	result = ROUTINE_INFO(/UNRESOLVED)

	void = WHERE(result eq 'L_PLOT', count)

	; If the L_PLOT routine has not been resolved find insight.sav
	; and restore it.
	;
	if (count ne 0) then begin

		; Look for insight in its default location idlxx/lib/hook
		;
		insightPath = FILEPATH('insight.sav', SUBDIR=['lib','hook'])
		void = FINDFILE(insightPath, COUNT=count)

		; If it wasn't found, notify the user.
		;
		if (count eq 0) then begin

			result = DIALOG_MESSAGE($
				['Insight was not found.',$
				'Please make sure insight.sav is in your hook directory', $
				'and !DIR points to your IDL directory.', $
				'Do you wish to locate insight.sav?'], /CANCEL)

			; If they want to search for it, display std file dialog.
			;
			if (result eq 'OK') then begin

				insightPath = $
					DIALOG_PICKFILE(TITLE='PLease locate insight.sav', $
						/MUST_EXIST)

				if (insightPath eq '') then $
					return

			endif else $
				return

		endif

		; Restore L_PLOT code
		;
		restore, insightPath
	endif

	; Call L_PLOT with passed in parameters
	;
	L_PLOT, y, _EXTRA=extra

end
